clear all;
close all;

% Mega test for dvbtenco
Ts = 7/64000000;
Fs = 1/Ts;

!dvbtenco -2k -M 64 -a 1 -i test.m2v -v 2 -o gluglu -d fvcprstmb

% First the MPEG-2 file
fid = fopen('test.m2v', 'r');
[A, len] = fread(fid, inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(len) ' MpegTs.']);
figure(1);
clf;
plot(A(1:1000));
title('MPEG-2 TS');

% Then the RS file
fid = fopen('gluglu_rs.bin', 'r');
[A, len] = fread(fid, inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(len) ' RS.']);
figure(2);
clf;
plot(A(1:1000));
title('RS codewords');

% Then the ConvInt file
fid = fopen('gluglu_ci.bin', 'r');
[A, len] = fread(fid, inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(len) ' ConvInt.']);
figure(3);
clf;
plot(A(1:1000));
title('Convolutional interleaved bytes');

% Then the ConvEnco file
fid = fopen('gluglu_ve.bin', 'r');
[A, len] = fread(fid, inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(len) ' ConvEnco.']);
figure(4);
clf;
plot(A(1:1000));
title('Convolutional encoded bits');

% Then the Punct file
fid = fopen('gluglu_pu.bin', 'r');
[A, len] = fread(fid, inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(len) ' Punct.']);
figure(5);
clf;
plot(A(1:1000));
title('Punctured bits');

% Then the BitInt file
fid = fopen('gluglu_bi.bin', 'r');
[A, len] = fread(fid, inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(len) ' BitInt.']);
figure(6);
clf;
plot(A(1:1000));
title('Bit interleaved bytes');

% Then the SymbInt file
fid = fopen('gluglu_si.bin', 'r');
[A, len] = fread(fid, inf, 'uchar');
fclose(fid);
disp(['Read ' num2str(len) ' SymbInt.']);
figure(7);
clf;
plot(A(1:1000));
title('Symbol interleaved bytes');

% Then the Map file
fid = fopen('gluglu_ma.bin', 'r');
[A, len] = fread(fid, inf, 'float32');
fclose(fid);
B = A(1:2:end) + j.*A(2:2:end);
disp(['Read ' num2str(length(B)) ' Map.']);
figure(8);
clf;
plot(B(1:1000), 'o');
title('Scatter plot of data carriers');
xlabel('Real part');
ylabel('Imaginary part');
figure(9);
clf;
subplot(211);
plot(real(B(1:1000)));
title('Mapped symbols');
xlabel('Inphase');
subplot(212);
plot(imag(B(1:1000)));
xlabel('Quadrature');

% Then the FreqDom file
fid = fopen('gluglu_fd.bin', 'r');
[A, len] = fread(fid, inf, 'float32');
fclose(fid);
B = A(1:2:end) + j.*A(2:2:end);
disp(['Read ' num2str(length(B)) ' FreqDom.']);
figure(10);
clf;
plot(B(1:1000), 'o');
title('Scatter plot of all carriers');
xlabel('Real part');
ylabel('Imaginary part');
figure(11);
clf;
subplot(211);
plot(real(B));
title('Frequency domain');
xlabel('Real')
subplot(212);
plot(imag(B));
xlabel('Imaginary')

% Then the TimeDom file
fid = fopen('gluglu.bin', 'r');
[A, len] = fread(fid, inf, 'float32');
fclose(fid);
B = A(1:2:240000) + j.*A(2:2:240000);
%B = A(1:2:end) + j.*A(2:2:end);
powB = (B' * B) / length(B);
%B = B ./ sqrt(powB);
disp(['Read ' num2str(length(B)) ' TimeDom.']);
figure(12);
clf;
plot(B(1:30000), '.');
title('Scatter plot of time domain samples');
xlabel('Real part');
ylabel('Imaginary part');
figure(13);
clf;
subplot(211);
plot(Ts * [0:29999] * 1e6, real(B(1:30000)));
title('Time domain')
ylabel('Inphase')
xlabel('Time (\mus)');
subplot(212);
plot(Ts * [0:29999] * 1e6, imag(B(1:30000)));
ylabel('Quadrature')
xlabel('Time (\mus)');
[Pxx, f] = psd(B, 512, Fs / 1000000, boxcar(512), 32);
figure(14);
clf;
plot(f - Fs/2000000, 10*log10(fftshift(Pxx)));
grid on;
title('Power Spectrum Density of \itdvbtenco\rm output');
xlabel('Frequency (MHz)');
ylabel('PSD (dB_c/Hz)');
figure(15);
clf;
zigma = sqrt((1512*1+176*16/9+17*1)/2);
%zigma = sqrt((6048*1+701*16/9+68*1)/2);
domain = linspace(-3*zigma, 3*zigma, 100);
gausso = (2.8/(sqrt(2*pi)*zigma)).*exp(-domain.^2/(2.*zigma.^2));
subplot(211);
[prob, dom] = hist(real(B), 100);
plot(dom, prob / length(B), 'b-', domain, gausso, 'r-');
title('Probability distribution function for I and Q components');
xlabel('I amplitude');
ylabel('%');
legend('Estimated', 'Theory');
subplot(212);
[prob, dom] = hist(imag(B), 100);
plot(dom, prob / length(B), 'b-', domain, gausso, 'r-');
xlabel('Q amplitude');
ylabel('%');
